@@ -5,6 +5,7 @@ import requests
 import sys
 import sqlite3
 from bs4 import BeautifulSoup
+from functools import wraps
 
 """
 Cambridge is a terminal version of Cambridge Dictionary. 
@@ -53,7 +54,6 @@ def list_words(args, con, cur):
             for i in data:
                 print(i[0])
 
-
 def search_words(args, con, cur):
     global REQUEST_URL
     global RESPONSE_WORD
@@ -83,14 +83,13 @@ def search_words(args, con, cur):
                     insert_into_table(con, cur, input_word, RESPONSE_WORD, RESPONSE_URL,RESPONSE_TEXT)
                 # If a user inputs a word with plural form, it won't get cached again
                 except sqlite3.IntegrityError:
-                    logger.debug("Not to cache word of '%s' because it already exists", RESPONSE_WORD)
+                    logger.debug("Cancel caching word of '%s' because it already exists", RESPONSE_WORD)
                 else:
                     logger.debug("Done caching search result of '%s'", input_word)
         else:
             logger.debug("Already cached '%s' before. Use cache", input_word)
             RESPONSE_URL, RESPONSE_TEXT = data[0], data[1]
-            print_dict()
-
+            print_dict(REQUEST_URL)
 
 def parse_args():
     parser = argparse.ArgumentParser(
@@ -199,9 +198,10 @@ def fetch(url, session):
         except Exception as e:
             attempt = call_on_error(e, url, attempt, OP[0])
         else:
+            if r.status_code != 200:  # only a 200 response has a response body
+                attempt = call_on_error(r.status_code, url, attempt, OP[0])
             return r
 
-
 def call_on_error(error, url, attempt, op):
     attempt += 1
     logger.debug(
@@ -216,9 +216,8 @@ def call_on_error(error, url, attempt, op):
         sys.exit()
     return attempt
 
-
 def parse_spellcheck(args, session):
-    word = " ".join(args.words).upper()
+    word = " ".join(args.words)
     query_word = "+".join(args.words)
     url = SPELLCHECK_BASE_URL + query_word
     response = fetch(url, session)
@@ -226,7 +225,7 @@ def parse_spellcheck(args, session):
     content = soup.find("div", "hfl-s lt2b lmt-10 lmb-25 lp-s_r-20")
     print()
     console.print("[bold #3C8DAD on #DDDDDD]" + word)
-    title = content.h1.text.upper().strip()
+    title = content.h1.text.strip()
     console.print("[bold]" + "\n" + title)
     for ul in content.find_all("ul", "hul-u"):
         notice = ul.find_previous_sibling().text
@@ -235,23 +234,32 @@ def parse_spellcheck(args, session):
             suggestion = replace_all(i.text)
             console.print("[#b2b2b2]" + "  • " + suggestion)
 
-
-def parse_first_dict():
+def parse_first_dict(url):
     global RESPONSE_TEXT
     global RESPONSE_URL
+    global RESPONSE_WORD
+
     if "isn’t in the Cambridge Dictionary yet. You can help!" in RESPONSE_TEXT or "404. Page not found" in RESPONSE_TEXT:
         print("\n" + str(NoResultError()) + "\n")
         sys.exit()
-    soup = BeautifulSoup(RESPONSE_TEXT, "lxml")
-    first_dict = soup.find("div", "pr dictionary")
-    if not first_dict:
-        print("\n" + str(ParsedNoneError()) + "\n")
-        sys.exit()
-    else:
-        return first_dict
 
-def parse_dict_blocks():
-    first_dict = parse_first_dict()
+    soup = BeautifulSoup(RESPONSE_TEXT, "lxml")
+    attempt = 0
+
+    while True:
+        try:
+            first_dict = soup.find("div", "pr dictionary")
+        except Exception as e:
+            attempt = call_on_error(e, url, attempt, OP[1])
+        else:
+            if not first_dict: 
+                attempt = call_on_error(ParsedNoneErrors(), url, attempt, OP[0])
+
+            RESPONSE_WORD = soup.find("title").text.split("|")[0].strip().lower()
+            return first_dict
+
+def parse_dict_blocks(url):
+    first_dict = parse_first_dict(url)
     try:
         result = first_dict.find(
             "div", ["pr entry-body__el", "entry-body__el clrd js-share-holder"]
@@ -271,7 +279,7 @@ def parse_dict_blocks():
 # ----------parse dict head----------
 def parse_head_title(title_block):
     if title_block.find("div", "di-title"):
-        word = title_block.find("div", "di-title").text.upper()
+        word = title_block.find("div", "di-title").text
         return word
 
 def parse_head_type(head):
@@ -285,10 +293,10 @@ def parse_head_type(head):
                 },
             ).text
         )
-        w_type = replace_all(w_type).upper()
+        w_type = replace_all(w_type)
     elif head.find("div", "posgram dpos-g hdib lmr-5"):
         posgram = head.find("div", "posgram dpos-g hdib lmr-5")
-        w_type = replace_all(posgram.text).upper()
+        w_type = replace_all(posgram.text)
     else:
         w_type = None
     return w_type
@@ -315,7 +323,7 @@ def parse_head_tense(head):
 
 
 def parse_head_domain(head):
-    domain = replace_all(head.find("span", "domain ddomain").text.upper())
+    domain = replace_all(head.find("span", "domain ddomain").text)
     console.print("[bold]" + domain, end="  ")
 
 
@@ -382,7 +390,7 @@ def parse_dict_head(block):
 
 # ----------parse dict body----------
 def parse_def_title(block):
-    d_title = replace_all(block.find("h3", "dsense_h").text).upper()
+    d_title = replace_all(block.find("h3", "dsense_h").text)
     console.print("[bold #3C8DAD]" + "\n" + d_title)
 
 
@@ -642,26 +650,26 @@ def parse_dict_name(first_dict):
     dict_info = replace_all(first_dict.small.text).replace("(", "").replace(")", "")
     dict_name = dict_info.split("©")[0]
     dict_name = dict_name.split("the")[-1]
-    console.print("[#125D98]" + dict_name + "\n", justify="right")
+    console.print(dict_name + "\n", justify="right")
 
 def request_and_print(url, args):
     with requests.Session() as session:
+        session.trust_env = False
         response = fetch(url, session)
         if response.url == DICT_BASE_URL:
             parse_spellcheck(args, session)
             return False
         else:
-            global RESPONSE_WORD
             global RESPONSE_URL
             global RESPONSE_TEXT
-            RESPONSE_URL, RESPONSE_WORD = parse_from_url(response.url)
+            RESPONSE_URL = parse_from_url(response.url)
             RESPONSE_TEXT = response.text
             logger.debug("Fetched html from %s successfully", RESPONSE_URL)
-            print_dict()
+            print_dict(url)
             return True
 
-def print_dict():
-    blocks, first_dict = parse_dict_blocks()
+def print_dict(url):
+    blocks, first_dict = parse_dict_blocks(url)
     for block in blocks:
         parse_dict_head(block)
         parse_dict_body(block)
@@ -669,6 +677,18 @@ def print_dict():
 
 
 # ----------main----------
+def timer(func):
+    @wraps(func)
+    def wrapper(*args, **kwargs):
+        start_at = time.time()
+        f = func(*args, **kwargs)
+        time_taken = round((time.time() - start_at), 2)
+        print("Time taken: {} seconds".format(time_taken))
+        return f
+
+    return wrapper
+
+# @timer
 def main():
     try:
         con = sqlite3.connect(DB, detect_types=sqlite3.PARSE_DECLTYPES | sqlite3.PARSE_COLNAMES)
@@ -691,10 +711,7 @@ if __name__ == "__main__":
     from errors import ParsedNoneError, NoResultError
     from cache import DB, create_table, insert_into_table, get_cache, get_response_words, get_random_words, delete_word, check_table
 
-    t1 = time.time()
     main()
-    t2 = time.time()
-    print(f"\nTIME TAKEN: {t2 - t1}")
 
 
 else:
